# GNN Cleanup

**GNN Cleanup** is a tool designed to improve tracking performance in the Belle II experiment by filtering background hits in the Central Drift Chamber (CDC) using a Graph Neural Network (GNN)-based approach. By leveraging graph-based inference, the HitCleanupModule efficiently distinguishes between signal and background hits, enhancing tracking accuracy and computational efficiency, especially in high-background conditions.

## Features
- **Hit Cleanup Module**: Filters background hits in the CDC using GNN inference, improving robustness to high-luminosity environments.
  - **get CDC Hit Info Module**: Extracts CDC information on hit level.
  - **Graph Construction**: Builds graph representations of CDC hits, including nodes (e.g., wire positions) and edges (e.g., spatial-temporal relationships).
  - **Preprocessing**: Applies feature clipping, scaling, and shifting to prepare graph data.
  - **GNN-Based Filtering**: Uses pretrained GNN models to perform hit filtering.

- **Example Scripts**:
  - A ready-to-use script demonstrates the full pipeline, including cleanup and tracking evaluation.
  - Runs full tracking workflows, including track finding and fitting, with or without the cleanup step.
  - Outputs tracking metrics, such as fitting charge efficiency, fake rate, and clone rate.

## Setup and Installation

1. **Clone the Repository**:
    ```bash
    git clone git@gitlab.etp.kit.edu:grheine/gnn_cleanup.git
    cd gnn_cleanup
    ```
3. **Run Setup Script**:
  Open your terminal and run:
   ```bash
   source setup.sh
   ```
   This sets up basf2 release 6, sources an virtual python environment and sets the PATH variables.

## Example Usage

To test the `gnn_cleanup`, run the example script with the appropriate options:

```bash
python3 examples/testing_withoutb2luigi.py --n 100
```

## Using other event files
For using other events first update the `testing_config.json` file. There you can specify the following settings:
- load_path: path to existing event root file
- expList: experiment number list for EventInfoSetter
- runList: run number list for EventInfoSetter
- bg_file_list_path: path to background file lists

You can either use existing event root files by setting the `load_path` to point to your files or generate new MC events by using the `--generate` flag and updating the `bg_file_list_path`, `expList` and `runList`.  

<!-- ## Contributing

We welcome contributions! Please see our [Contributing Guide](CONTRIBUTING.md) for more details on how to get started.

## License

This project is licensed under the MIT License. See the [LICENSE](LICENSE) file for details.

## Authors and acknowledgment

Developed by the Belle II GNN team. Special thanks to all contributors and the Belle II collaboration for their support.

## Project status

This project is actively maintained. Contributions and feedback are welcome. -->

<!-- # Editing this README

When you're ready to make this README your own, just edit this file and use the handy template below (or feel free to structure it however you want - this is just a starting point!). Thanks to [makeareadme.com](https://www.makeareadme.com/) for this template.

## Suggestions for a good README

Every project is different, so consider which of these sections apply to yours. The sections used in the template are suggestions for most open source projects. Also keep in mind that while a README can be too long and detailed, too long is better than too short. If you think your README is too long, consider utilizing another form of documentation rather than cutting out information.

## Name
Choose a self-explaining name for your project.

## Description
Let people know what your project can do specifically. Provide context and add a link to any reference visitors might be unfamiliar with. A list of Features or a Background subsection can also be added here. If there are alternatives to your project, this is a good place to list differentiating factors.

## Badges
On some READMEs, you may see small images that convey metadata, such as whether or not all the tests are passing for the project. You can use Shields to add some to your README. Many services also have instructions for adding a badge.

## Visuals
Depending on what you are making, it can be a good idea to include screenshots or even a video (you'll frequently see GIFs rather than actual videos). Tools like ttygif can help, but check out Asciinema for a more sophisticated method.

## Installation
Within a particular ecosystem, there may be a common way of installing things, such as using Yarn, NuGet, or Homebrew. However, consider the possibility that whoever is reading your README is a novice and would like more guidance. Listing specific steps helps remove ambiguity and gets people to using your project as quickly as possible. If it only runs in a specific context like a particular programming language version or operating system or has dependencies that have to be installed manually, also add a Requirements subsection.

## Usage
Use examples liberally, and show the expected output if you can. It's helpful to have inline the smallest example of usage that you can demonstrate, while providing links to more sophisticated examples if they are too long to reasonably include in the README.

## Support
Tell people where they can go to for help. It can be any combination of an issue tracker, a chat room, an email address, etc.

## Roadmap
If you have ideas for releases in the future, it is a good idea to list them in the README.

## Contributing
State if you are open to contributions and what your requirements are for accepting them.

For people who want to make changes to your project, it's helpful to have some documentation on how to get started. Perhaps there is a script that they should run or some environment variables that they need to set. Make these steps explicit. These instructions could also be useful to your future self.

You can also document commands to lint the code or run tests. These steps help to ensure high code quality and reduce the likelihood that the changes inadvertently break something. Having instructions for running tests is especially helpful if it requires external setup, such as starting a Selenium server for testing in a browser.

## Authors and acknowledgment
Show your appreciation to those who have contributed to the project.

## License
For open source projects, say how it is licensed.

## Project status
If you have run out of energy or time for your project, put a note at the top of the README saying that development has slowed down or stopped completely. Someone may choose to fork your project or volunteer to step in as a maintainer or owner, allowing your project to keep going. You can also make an explicit request for maintainers. -->
